/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/
using System;
using System.Collections.Generic;
using System.Web.Services;
using Egora.Pvp;
using Egora.Stammportal;
using Egora.Stammportal.LdapAuthorizationService;

[WebService(Namespace = CustomAuthorization.Namespace)]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class LdapAuthorizer : WebService, IAuthorizationContract
{
  /// <summary>
  /// Gets the authorization information for a given application and user
  /// </summary>
  /// <param name="rootUrl">RootUrl of Application</param>
  /// <param name="userId">Unique identifier of a user, e.g. loginname</param>
  /// <returns>HttpHeaders for ApplicationType Web, SOAP Header for ApplcationType SOAP</returns>
  [WebMethod]
  public CustomAuthorization GetAuthorization(string rootUrl, string userId)
  {
    LdapConfiguration configuration = LdapConfiguration.GetConfiguration();

    rootUrl = rootUrl.ToLowerInvariant();
    PvpApplicationLdapAuthorizer authorizer = new PvpApplicationLdapAuthorizer(rootUrl, userId, configuration);

    if (!authorizer.IsValid)
      return null;

    CustomAuthorization auth = new CustomAuthorization();
    auth.TimeToLive = authorizer.AuthorizationTimeToLive;

    if (authorizer.IsWeb)
    {
      List<HttpHeader> headers = new List<HttpHeader>();
      AddIfNotEmpty(headers, PvpIdentifier.X_Version, authorizer.Version);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_participantId, authorizer.ParticipantID);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_UserId, userId.Replace(@"\", "_"));
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_cn, authorizer.CommonName);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_gvGid, authorizer.GvGID);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_gvOuId, authorizer.GvOuID);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_gvOuDomain, authorizer.GvOuDomain);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_Ou, authorizer.Ou);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_gvFunction, authorizer.GvFunction);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_mail, authorizer.Mail);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_tel, authorizer.Telephone);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHENTICATE_gvSecClass, authorizer.GvSecClass);
      //if (authorizer.ApplicationName != "zgw" ) Ausnahme fr Zentrales Gewerberegister, keine Rolle wird gesendet
        AddIfNotEmpty(headers, PvpIdentifier.X_AUTHORIZE_roles, authorizer.Roles);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHORIZE_gvOuId, authorizer.AuthorizeGvOuID);
      AddIfNotEmpty(headers, PvpIdentifier.X_AUTHORIZE_Ou, authorizer.AuthorizeOU);
      AddIfNotEmpty(headers, PvpIdentifier.X_ACCOUNTING_InvoiceRecptId, authorizer.InvoiceRecptId);
      AddIfNotEmpty(headers, PvpIdentifier.X_ACCOUNTING_CostCenterId, authorizer.CostCenterId);

      auth.HttpHeaders = headers.ToArray();
    }

    else if (authorizer.IsSoap)
    {
      auth.SoapHeaderXmlFragment = authorizer.PvpSoapFragment;
    }
    else
    {
      auth = CustomAuthorization.NoAuthorization;
    }

    return auth;
  }

  private void AddIfNotEmpty(List<HttpHeader> headers, string headerName, string headerValue)
  {
    if (!String.IsNullOrEmpty(headerValue))
      headers.Add(new HttpHeader(headerName, headerValue));
  }
}