/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/
using System.Net;
using System.Xml.Serialization;
using NUnit.Framework;

namespace Egora.Stammportal.HttpReverseProxy.UnitTests.IntegrationTest
{
  [TestFixture]
  public class HeaderTest
  {
    [Test]
    public void CustomHeader()
    {
      HttpWebRequest request = CreateRequest();
      request.Headers.Add("X-Custom", "SomeValue");
      HttpWebResponse response = (HttpWebResponse) request.GetResponse();

      Assert.IsNotNull(response, "Response");

      XmlSerializer serializer = new XmlSerializer(typeof (RequestInformation));
      RequestInformation info = (RequestInformation) serializer.Deserialize(response.GetResponseStream());

      Assert.IsNotNull(info, "RequestInformation");
      Assert.AreEqual("SomeValue", info.GetHeader("X-Custom"));

      Assert.IsNotNull(info.GetHeader("PVP-Header1"), "PVP-Header1");
      Assert.AreEqual("PVP-Value1", info.GetHeader("PVP-Header1"));
    }

    [Test]
    public void FromHeader()
    {
      HttpWebRequest request = CreateRequest();
      request.Headers.Add("From", "Hugo");
      HttpWebResponse response = (HttpWebResponse)request.GetResponse();

      Assert.IsNotNull(response, "Response");

      XmlSerializer serializer = new XmlSerializer(typeof(RequestInformation));
      RequestInformation info = (RequestInformation)serializer.Deserialize(response.GetResponseStream());

      Assert.IsNotNull(info, "RequestInformation");
      Assert.IsNull(info.GetHeader("From")); // because header is removed
      Assert.AreEqual("Hugo", info.GetHeader("X-AUTHENTICATE-UserId"));

      Assert.IsNotNull(info.GetHeader("PVP-Header1"), "PVP-Header1");
      Assert.AreEqual("PVP-Value1", info.GetHeader("PVP-Header1"));
    }

    private HttpWebRequest CreateRequest()
    {
      HttpWebRequest request =
        (HttpWebRequest)WebRequest.Create("http://egoratest/Stammportal/localtest/IntegrationTestPage.aspx");
      request.UseDefaultCredentials = true;
      return request;
    }
  }
}