/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/

using System;
using System.Collections.Generic;
using System.Net;
using System.Web;

namespace Egora.Stammportal.HttpReverseProxy
{
  // folgende Punkt offen:
  // 1) die meisten Browser untersttzen 20 Cookies pro Domain
  // 2) wenn Secure gesetzt ist, und Client kommt mit http, dann geht Cookie verloren
  // 3) im Value werden vorne domain und path dazugenommen, dadurch kann hinten was verloren gehen
  public class CookieTransformer
  {
    public const string c_CookieSignature = "CD9C0F68-2099-4f51-A585-E9758EF0A020";
    private string _remoteApplicationProxyPath;

    public CookieTransformer(string remoteApplicationProxyPath)
    {
      if (remoteApplicationProxyPath == null)
        throw new ArgumentException("remoteApplicationProxyPath must not be null.");

      _remoteApplicationProxyPath = remoteApplicationProxyPath;
    }

    public virtual HttpCookie[] LeftSideResponseCookies(CookieCollection rightSideResponseCookies)
    {
      if (_remoteApplicationProxyPath == null)
        throw new ApplicationException("Transformation of ResponseCookies requires RemoteApplicationProxyPath.");

      List<HttpCookie> leftSideResponseCookies = new List<HttpCookie>();

      foreach (Cookie rightSideResponseCookie in rightSideResponseCookies)
        leftSideResponseCookies.Add(LeftSideResponseCookie(rightSideResponseCookie));

      return leftSideResponseCookies.ToArray();
    }

    private string CookieNamePrefix
    {
      get { return _remoteApplicationProxyPath; }
    }

    public virtual HttpCookie LeftSideResponseCookie(Cookie rightSideResponseCookie)
    {
      string value = String.Format("{0}|{1}|{2}|{3}", c_CookieSignature, rightSideResponseCookie.Domain,
                                   rightSideResponseCookie.Path, rightSideResponseCookie.Value);
      HttpCookie newCookie = new HttpCookie(CookieNamePrefix + rightSideResponseCookie.Name, value);
      newCookie.Expires = rightSideResponseCookie.Expires;
      newCookie.HttpOnly = rightSideResponseCookie.HttpOnly;
      newCookie.Secure = rightSideResponseCookie.Secure;
      newCookie.Path = _remoteApplicationProxyPath;
      return newCookie;
    }

    public Cookie RightSideRequestCookie(HttpCookie leftSideRequestCookie)
    {
      // leftSideRequestCookie.Path is always "/"
      // if (String.Compare (leftSideRequestCookie.Path, _remoteApplicationProxyPath, true) != 0)
      //  // not my cookie
      //  return null;

      string[] infos = leftSideRequestCookie.Value.Split(new char[] {'|'}, 4);
      if (infos.Length < 4)
        // not my cookie
        return null;

      string signature = infos[0];
      if (signature != c_CookieSignature)
        return null;

      string cookiePath = infos[2];
      //if (cookiePath == String.Empty || cookiePath == "/")
      //  // not my cookie
      //  return null;

      int prefixLength = CookieNamePrefix.Length;
      if (leftSideRequestCookie.Name.Length < prefixLength)
        return null; // can not handle this cookie

      Cookie rightSideCookie = new Cookie(leftSideRequestCookie.Name.Remove(0, prefixLength), infos[3]);
      if (!String.IsNullOrEmpty(infos[1]))
        rightSideCookie.Domain = infos[1];
      rightSideCookie.Path = cookiePath;
      rightSideCookie.Expires = leftSideRequestCookie.Expires;
      rightSideCookie.HttpOnly = leftSideRequestCookie.HttpOnly;
      rightSideCookie.Secure = leftSideRequestCookie.Secure;
      return rightSideCookie;
    }

    public CookieCollection RightSideRequestCookies(HttpRequest leftSideRequest)
    {
      CookieCollection cookies = new CookieCollection();
      foreach (string leftSideCookieName in leftSideRequest.Cookies)
      {
        Cookie rightSideRequestCookie = RightSideRequestCookie(leftSideRequest.Cookies[leftSideCookieName]);
        if (rightSideRequestCookie != null)
          cookies.Add(rightSideRequestCookie);
      }
      return cookies;
    }
  }
}