/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/
using System;
using System.Collections.Generic;
using System.DirectoryServices;
using System.Xml.Serialization;

namespace Egora.Stammportal.LdapAuthorizationService
{
  public enum PropertySource
  {
    Undefined = 0,
    User = 1,
    Group = 2,
    UserOrGroup = 3,
  }

  public class PvpAttribute
  {
    private string _name;

    [XmlAttribute("name")]
    public string Name
    {
      get { return _name; }
      set { _name = value; }
    }

    private string _ldapAttribute;

    [XmlAttribute("ldapAttribute")]
    public string LdapAttribute
    {
      get
      {
        if (_ldapAttribute != null)
          return _ldapAttribute;
        if (GlobalAttribute != null)
          return GlobalAttribute.LdapAttribute;
        return null;
      }
      set { _ldapAttribute = value; }
    }

    private string _format;

    [XmlAttribute("format")]
    public string Format
    {
      get
      {
        if (_format != null)
          return _format;
        if (GlobalAttribute != null)
          return GlobalAttribute.Format;
        return "{0}";
      }

      set { _format = value; }
    }

    private string _formatter;
    [XmlAttribute("formatter")]
    public string Formatter
    {
      get
      {
        if (_formatter != null)
          return _formatter;
        if (GlobalAttribute != null)
          return GlobalAttribute.Formatter;
        return null;
      }

      set { _formatter = value; }
    }

    private string _defaultValue;

    [XmlAttribute("default")]
    public string DefaultValue
    {
      get
      {
        if (_defaultValue != null)
          return _defaultValue;
        if (GlobalAttribute != null)
          return GlobalAttribute.DefaultValue;
        return null;
      }

      set { _defaultValue = value; }
    }

    private PropertySource _source = PropertySource.Undefined;

    [XmlAttribute("source")]
    public PropertySource Source
    {
      get
      {
        if (_source != PropertySource.Undefined)
          return _source;
        if (GlobalAttribute != null)
          return GlobalAttribute.Source;
        return PropertySource.User;
      }
      set { _source = value; }
    }

    private PvpAttribute _globalAttribute;

    [XmlIgnore]
    internal PvpAttribute GlobalAttribute
    {
      get { return _globalAttribute; }
      set { _globalAttribute = value; }
    }

    public string[] GetValue(DirectoryEntry user, List<DirectoryEntry> groups)
    {
      if (user == null)
        return null;

      if (Source == PropertySource.User)
      {
        string propValue = GetEntryValue(user);
        if (propValue != null)
          return new string[] {FormatValue(propValue)};
        return new string[] {DefaultValue};
      }

      if (Source == PropertySource.UserOrGroup)
      {
        string propValue = GetEntryValue(user);
        if (propValue != null)
          return new string[] {FormatValue(propValue)};

        return GetGroupValue(groups);
      }

      if (Source == PropertySource.Group)
        return GetGroupValue(groups);

      return null;
    }

    private string FormatValue(string propValue)
    {
      if (String.IsNullOrEmpty(Formatter))
        return String.Format(Format, propValue);

      Type formatterType = Type.GetType(Formatter);
      IValueFormatter formatter = Activator.CreateInstance(formatterType) as IValueFormatter;
      if (formatter == null)
        return String.Format(Format, propValue);
      
      return formatter.Format(propValue);
    }

    private string GetEntryValue(DirectoryEntry entry)
    {
      if (String.IsNullOrEmpty(LdapAttribute))
        return null;

      string propValue = null;
      if (entry.Properties.Contains(LdapAttribute))
        propValue = entry.Properties[LdapAttribute].Value.ToString();

      return propValue;
    }

    private string[] GetGroupValue(List<DirectoryEntry> groups)
    {
      List<string> values = new List<string>();
      foreach (DirectoryEntry group in groups)
      {
        values.Add(FormatValue(GetEntryValue(group)));
      }
      if (values.Count == 0)
        values.Add(DefaultValue);

      return values.ToArray();
    }
  }
}