/*************************
Diese Software ist ein Beispiel (sample code) und unterliegt der Microsoft Public License. 
Die Verwendung des Codes ist unter den Bedingungen der Microsoft Public License erlaubt.
*************************
This software is sample code and is subject to the Microsoft Public License. 
You may use this code according to the conditions of the Microsoft Public License.
*************************/

using System.Collections.Specialized;
using System.Security.Principal;
using Egora.Pvp;

namespace Egora.PvpHttpModule
{
  public class PvpPrincipal : IPvpPrincipal
  {
    private readonly NameValueCollection _headers;
    private readonly PvpIdentity _identity;
    private PvpAuthorization _authorization;


    public PvpPrincipal(NameValueCollection headers)
    {
      _headers = headers;
      _identity = new PvpIdentity(_headers);
      _authorization = new PvpAuthorization(_headers);
    }

    /// <summary>
    ///                     Determines whether the current principal belongs to the specified role.
    /// </summary>
    /// <returns>
    /// true if the current principal is a member of the specified role; otherwise, false.
    /// </returns>
    /// <param name="roleName">
    ///                     The name of the role for which to check membership. 
    ///                 </param>
    public bool IsInRole(string roleName)
    {
      return _authorization.HasRole(roleName);
    }

    public bool IsInRole(string roleName, NameValueCollection roleParameters)
    {
      return _authorization.HasRole(roleName, roleParameters);
    }

    /// <summary>
    ///                     Gets the identity of the current principal.
    /// </summary>
    /// <returns>
    ///                     The <see cref="T:System.Security.Principal.IIdentity" /> object associated with the current principal.
    /// </returns>
    public IIdentity Identity
    {
      get { return _identity; }
    }

    public string GvOuId
    {
      get { return _authorization.GvOuId; }
      set { throw new System.NotImplementedException(); }
    }

    public string Ou
    {
      get { return _authorization.Ou; }
      set { throw new System.NotImplementedException(); }
    }

    public virtual string GetHeader(string key)
    {
      return _headers[key];
    }

    public virtual string GetHeader(PvpHeader header)
    {
      return _headers[PvpHeaderNames.GetHeaderName(header)];
    }

    public IPvpRole[] GetRoles()
    {
      return (_authorization.Roles == null) ? null : _authorization.Roles.ToArray();
    }

    public PvpAuthorization GetAuthorization()
    {
      return _authorization;
    }

    public IPvpRole GetRole(string name)
    {
      return _authorization.GetRole(name);
    }
  }
}